<?php

namespace FiloBlu\Refilo\Model;

use Exception;
use FiloBlu\Newsletter\Model\SubscriberInfoFactory;
use FiloBlu\Refilo\Api\NewsletterManagementInterface;
use Magento\Customer\Api\AccountManagementInterface as CustomerAccountManagement;
use Magento\Customer\Model\Customer;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Module\Manager;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Validator\EmailAddress;
use Magento\Newsletter\Model\Subscriber;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Store\Api\StoreRepositoryInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Throwable;
use Zend_Validate;

/**
 * {@inheritDoc}
 */
class NewsletterManagement implements NewsletterManagementInterface
{
    /**
     * @var ObjectManagerInterface
     */
    protected $_objectManager;

    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var Session
     */
    protected $_customerSession;

    /**
     * @var CustomerAccountManagement
     */
    protected $customerAccountManagement;

    /**
     * @var SubscriberFactory
     */
    protected $_subscriberFactory;
    /**
     * @var SubscriberInfoFactory
     */
    protected $_subscriber_info;

    /**
     * @var Customer
     */
    protected $_customer;

    /**
     * @var Manager
     */
    protected $_moduleManager;
    /**
     * @var Manager
     */
    protected $moduleManager;
    /**
     * @var StoreRepositoryInterface
     */
    private $storeRepository;
    /**
     * @var Context
     */
    private $context;

    /**
     * NewsletterManagement constructor.
     * @param Context $context
     * @param StoreManagerInterface $storeManager
     * @param Session $customerSession
     * @param CustomerAccountManagement $customerAccountManagement
     * @param SubscriberFactory $subscriberFactory
     * @param Customer $_customer
     * @param SubscriberInfoFactory $subscriber_info
     * @param Manager $moduleManager
     * @param StoreRepositoryInterface $storeRepository
     */
    public function __construct(
        Context $context,
        StoreManagerInterface $storeManager,
        Session $customerSession,
        CustomerAccountManagement $customerAccountManagement,
        SubscriberFactory $subscriberFactory,
        Customer $_customer,
        SubscriberInfoFactory $subscriber_info,
        Manager $moduleManager,
        StoreRepositoryInterface $storeRepository
    ) {
        $this->_objectManager = $context->getObjectManager();
        $this->_storeManager = $storeManager;
        $this->_customerSession = $customerSession;
        $this->customerAccountManagement = $customerAccountManagement;
        $this->_subscriberFactory = $subscriberFactory;
        $this->_customer = $_customer;
        $this->_subscriber_info = $subscriber_info;
        $this->moduleManager = $moduleManager;
        $this->storeRepository = $storeRepository;
        $this->context = $context;
    }

    /**
     * @param string|null $email
     * @param string|null $source
     * @param string|null $firstname
     * @param string|null $lastname
     * @param string|null $dob
     * @param string|null $gender
     * @param string|null $country
     * @param string|null $collections
     * @param string|null $phone
     * @param string|null $language
     * @param string|null $website_name
     * @param null $store
     * @return \Magento\Framework\DataObject
     */
    public function subscribe(
        $email,
        $source,
        $firstname = null,
        $lastname = null,
        $dob = null,
        $gender = null,
        $country = null,
        $collections = null,
        $phone = null,
        $language = null,
        $website_name = null,
        $store = null
    ) {
        $success = false;

        $subscriber = null;
        try {
            $storeByCode = $this->storeRepository->getActiveStoreByCode($store);
            $this->validateEmailFormat($email);
            $this->validateGuestSubscription();

            $subscriber = $this->_subscriberFactory->create()->loadByEmail($email);
            if ($subscriber->getId()
                && $subscriber->getSubscriberStatus() == Subscriber::STATUS_SUBSCRIBED
            ) {
                throw new LocalizedException(
                    __('This email address is already subscribed.')
                );
            }

            $subscriber->setStoreId($storeByCode->getId());

            $status = $subscriber->subscribe($email);
            if ($status == Subscriber::STATUS_NOT_ACTIVE) {
                $success = true;
                $message = __('The confirmation request has been sent.');
            } else {
                $success = true;
                $message = __('Thank you for your subscription.');
            }
        } catch (LocalizedException $e) {
            $message = __('There was a problem with the subscription: %1', $e->getMessage());
        } catch (Exception $e) {
            $message = __('There was a problem with the subscription: %1', $e->getMessage());
        }

        try {
            if ($email) {
                $data = [
                    'subscriber_id' => $subscriber->getId(),
                    'source'        => $source,
                    'firstname'     => $firstname,
                    'lastname'      => $lastname,
                    'dob'           => $dob,
                    'country'       => $country,
                    'gender'        => $gender,
                    'collections'   => $collections,
                    'language'      => $language,
                    'website_name'  => $website_name,
                    'phone'         => $phone
                ];
                $this->setSubscriberInfo($data);
            }
        } catch (Throwable $e) {
            $message = __('There was a problem with the subscription: %1', $e->getMessage());
        }

        $data = [
            'success' => $success,
            'message' => $message,
        ];

        $result = new DataObject();
        $result->setData($data);
        return $result;
    }

    /**
     * Validates the format of the email address
     * Reference: vendor/magento/module-newsletter/Controller/Subscriber/NewAction.php
     *
     * @param string $email
     * @return void
     * @throws LocalizedException
     */
    protected function validateEmailFormat($email)
    {
        if (!Zend_Validate::is($email, EmailAddress::class)) {
            throw new LocalizedException(__('Please enter a valid email address.'));
        }
    }

    /**
     * Validates that if the current user is a guest, that they can subscribe to a newsletter.
     * Reference: vendor/magento/module-newsletter/Controller/Subscriber/NewAction.php
     *
     * @return void
     * @throws LocalizedException
     */
    protected function validateGuestSubscription()
    {
        if ($this->_objectManager->get(ScopeConfigInterface::class)
                ->getValue(
                    Subscriber::XML_PATH_ALLOW_GUEST_SUBSCRIBE_FLAG,
                    ScopeInterface::SCOPE_STORE
                ) != 1
        ) {
            throw new LocalizedException(
                __(
                    'subscription denied for guests.'
                )
            );
        }
    }

    public function setSubscriberInfo($data)
    {
        if ($this->moduleManager->isEnabled('FiloBlu_Newsletter')) {
            $subscribe = $this->_subscriber_info->create();
            $subscribe->addData($data);
            $subscribe->save();
        }
    }

    /**
     * Validates that the email address isn't being used by a different account.
     * Reference: vendor/magento/module-newsletter/Controller/Subscriber/NewAction.php
     *
     * @param string $email
     * @return void
     * @throws LocalizedException
     */
    protected function validateEmailAvailable($email)
    {
        $websiteId = $this->_storeManager->getStore()->getWebsiteId();

        if (!$this->customerAccountManagement->isEmailAvailable($email, $websiteId)) {
            throw new LocalizedException(
                __('This email address is already assigned to another user.')
            );
        }
    }
}
